import {
  Body,
  Controller,
  Delete,
  Get,
  Post,
  Query,
  Request,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { CarreraProgramaService } from './carrera-programa.service';
import { ValidarUsuarioService } from '../validar-usuario/validar-usuario.service';
import { Operador } from '../operador/entity/operador.entity';
import { Usuario } from '../usuario/entity/usuario.entity';
import { IdInstitucionDto } from '../dto/input/id-institucion.dto';
import { CreateCarreraProgramaDto } from './dto/input/create.dto';
import { DeleteCarreraProgramaDto } from './dto/input/delete.dto';
import { MessageOutputDto } from '../dto/output/message.dto';
import { CarreraProgramaOutputDto } from './dto/output/carrera-programa.dto';

@Controller('carrera-programa')
@ApiTags('carrera-programa')
export class CarreraProgramaController {
  constructor(
    private carreraProgramaService: CarreraProgramaService,
    private validarUsuarioService: ValidarUsuarioService,
  ) {}

  @Serealize(MessageOutputDto)
  @Post()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Asigna un programa a una carrera.' })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Variables que necesita el endpoint.',
    examples: {
      ejemplo: { value: { id_institucion_carrera: 36, id_programa: 1 } },
    },
  })
  create(@Request() req, @Body() body: CreateCarreraProgramaDto) {
    const admin: Operador = req.user.operador;

    this.validarUsuarioService.validarAdmin(admin);
    return this.carreraProgramaService.create(
      admin,
      body.id_institucion_carrera,
      body.id_programa,
    );
  }

  @Serealize(MessageOutputDto)
  @Delete()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Eliminar asociación entre un programa y una carrera.',
  })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Variables que necesita el endpoint.',
    examples: { ejemplo: { value: { id_carrera_programa: 1 } } },
  })
  delete(@Request() req, @Body() body: DeleteCarreraProgramaDto) {
    const admin: Operador = req.user.operador;

    this.validarUsuarioService.validarAdmin(admin);
    return this.carreraProgramaService.delete(admin, body.id_carrera_programa);
  }

  @Serealize(CarreraProgramaOutputDto)
  @Get()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Programas asignados a las carreras de una institución de un admin.',
  })
  @ApiBearerAuth('jwt')
  get(@Request() req) {
    const admin: Operador = req.user.operador;

    this.validarUsuarioService.validarAdmin(admin);
    return this.carreraProgramaService.findAllByInstitucion(admin.institucion);
  }

  @Serealize(CarreraProgramaOutputDto)
  @Get('usuario')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Programas asignados a las carreras de una institución de un usuario.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  getUsuario(@Request() req, @Query() query: IdInstitucionDto) {
    const usuario: Usuario = req.user.usuario;

    this.validarUsuarioService.validarUsuario(usuario);
    return this.carreraProgramaService.findAllOfUsuario(
      usuario,
      parseInt(query.id_institucion),
    );
  }
}
