import * as moment from 'moment';
import {
  BadRequestException,
  ConflictException,
  ForbiddenException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Institucion } from '../institucion/entity/institucion.entity';
import { InstitucionDia } from './entity/institucion-dia.entity';
import { Usuario } from 'src/usuario/entity/usuario.entity';
import { Operador } from '../operador/entity/operador.entity';
import { InstitucionService } from '../institucion/institucion.service';
import { InstitucionUsuarioService } from '../institucion-usuario/institucion-usuario.service';

@Injectable()
export class InstitucionDiaService {
  constructor(
    @InjectRepository(InstitucionDia)
    private repository: Repository<InstitucionDia>,
    private institucionService: InstitucionService,
    private institucionUsuarioService: InstitucionUsuarioService,
  ) {}

  findAllAdmin(admin: Operador): Promise<InstitucionDia[]> {
    return this.repository.find({
      where: { institucion: admin.institucion },
      order: { id_institucion_dia: 'ASC' },
    });
  }

  findById(id_institucion_dia: number): Promise<InstitucionDia> {
    return this.repository
      .findOne({
        join: {
          alias: 'id',
          innerJoinAndSelect: { d: 'id.dia', i: 'id.institucion' },
        },
        where: { id_institucion_dia },
      })
      .then((institucionDia) => {
        if (!institucionDia)
          throw new NotFoundException('No existe este id institución día.');
        return institucionDia;
      });
  }

  hoy(institucion: Institucion): Promise<InstitucionDia> {
    const ahora = moment();

    // Validamos que no sea domingo
    if (ahora.weekday() === 0)
      throw new ConflictException('No hay servicio los días domingo.');
    return this.repository.findOne({
      where: { institucion, dia: { id_dia: ahora.weekday() } },
    });
  }

  hoyUsuario(
    usuario: Usuario,
    id_institucion: number,
  ): Promise<InstitucionDia> {
    return this.institucionService
      .findById(id_institucion)
      .then(async (institucion) => {
        const institucionUsuario =
          await this.institucionUsuarioService.findByIdUsuarioIdInstitucion(
            usuario,
            institucion,
          );

        // Validamos que el usuario pertenezca a la institución de la información que solicita
        if (!institucionUsuario)
          throw new ForbiddenException(
            'No puedes acceder a esta información porque no peteneces a esta institución.',
          );
        return this.hoy(institucion);
      });
  }

  update(
    admin: Operador,
    attrs: Partial<InstitucionDia>,
  ): Promise<{ message: string }> {
    return this.findById(attrs.id_institucion_dia)
      .then((institucionDia) => {
        // Validamos que la institución día pertenezca a la institución del admin
        if (
          admin.institucion.id_institucion !=
          institucionDia.institucion.id_institucion
        )
          throw new ConflictException(
            'No puedes modificar la información este día porque no pertenece a tu institución.',
          );
        // Validamos que hora_inicio no sea mayor a hora_fin
        if (
          // Si se mando hora_inicio u hora_fin
          (attrs.hora_inicio || attrs.hora_fin) &&
          // y si la hora_inicio que se mando, si no hay toma de institucionDia, es mayor que
          (attrs.hora_inicio || institucionDia.hora_inicio) >
            // la hora_fin que se mando, si no hay toma de institucionDia
            (attrs.hora_fin || institucionDia.hora_fin)
        )
          throw new BadRequestException(
            'La hora inicio no puede ser mayor que la hora fin.',
          );
        // Validamos que hora_fin no sea mayor a hora_extra
        if (
          // Si se mando hora_fin u hora_extra
          (attrs.hora_fin || attrs.hora_extra) &&
          // y si la hora_fin que se mando, si no hay toma de institucionDia, es mayor que
          (attrs.hora_fin || institucionDia.hora_fin) >
            // la hora_extra que se mando, si no hay toma de institucionDia
            (attrs.hora_extra || institucionDia.hora_extra)
        )
          throw new BadRequestException(
            'La hora fin no puede ser mayor que la hora extra.',
          );
        // Validamos que hora_extra no sea mayor a hora_tope
        if (
          // Si se hora_extra hora_fin u hora_tope
          (attrs.hora_extra || attrs.hora_tope) &&
          // y si la hora_extra que se mando, si no hay toma de institucionDia, es mayor que
          (attrs.hora_extra || institucionDia.hora_extra) >
            // la hora_tope que se mando, si no hay toma de institucionDia
            (attrs.hora_tope || institucionDia.hora_tope)
        )
          throw new BadRequestException(
            'La hora extra no puede ser mayor que la hora tope.',
          );
        // Asignamos valores enviados al objeto
        Object.assign(institucionDia, attrs);
        // Guardamos cambios
        return this.repository.save(institucionDia);
      })
      .then((_) => ({ message: 'Se guardaron los cambios correctamente.' }));
  }
}
