import { ConflictException, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FindOptionsWhere, Repository } from 'typeorm';
import { Institucion } from '../institucion/entity/institucion.entity';
import { InstitucionCarrera } from '../institucion-carrera/entity/institucion-carrera.entity';
import { InstitucionUsuario } from './entity/institucion-usuario.entity';
import { Operador } from '../operador/entity/operador.entity';
import { Usuario } from '../usuario/entity/usuario.entity';
import { FullInformacionInstitucionUsuarioView } from './entity/views/full-informacion-institucion-usuario.view';
import { InformacionInstitucionUsuarioView } from './entity/views/informacion-institucion-usuario.view';

@Injectable()
export class InstitucionUsuarioService {
  constructor(
    @InjectRepository(InstitucionUsuario)
    private repository: Repository<InstitucionUsuario>,
    @InjectRepository(FullInformacionInstitucionUsuarioView)
    private fullInformacionInstitucionUsuarioView: Repository<FullInformacionInstitucionUsuarioView>,
    @InjectRepository(InformacionInstitucionUsuarioView)
    private informacionInstitucionUsuarioView: Repository<InformacionInstitucionUsuarioView>,
  ) {}

  create(
    institucionCarrera: InstitucionCarrera,
    usuario: Usuario,
  ): Promise<InstitucionUsuario> {
    // Creamos y guardamos un registro
    return this.repository.save(
      this.repository.create({ institucionCarrera, usuario }),
    );
  }

  desactivarCuentas(admin: Operador): Promise<{ message: string }> {
    // Buscamos todas los registros que le pertenecen a la institutción del admin
    return this.informacionInstitucionUsuarioView
      .find({
        where: { id_institucion: admin.institucion.id_institucion, activo: 1 },
      })
      .then(async (infoInstitucionesUsuario) => {
        // Recorremos el array
        for (let i = 0; i < infoInstitucionesUsuario.length; i++)
          // Desactivamos las cuentas y guardamos los cambios
          await this.repository.save(
            this.repository.create({
              id_institucion_usuario:
                infoInstitucionesUsuario[i].id_institucion_usuario,
              activo: false,
            }),
          );
        return { message: 'Se desactivaron todas las cuentas.' };
      });
  }

  findAllByIdUsuarioIdInstitucion(
    usuario: Usuario,
    institucion: Institucion,
  ): Promise<InstitucionUsuario[]> {
    return this.informacionInstitucionUsuarioView
      .find({
        where: {
          id_institucion: institucion.id_institucion,
          id_usuario: usuario.id_usuario,
        },
      })
      .then((infoInstitucionesUsuario) => {
        const institucionesUsuario: InstitucionUsuario[] = [];

        for (let i = 0; i < infoInstitucionesUsuario.length; i++)
          institucionesUsuario.push(
            this.viewToInstitucionUsuario(infoInstitucionesUsuario[i]),
          );
        return institucionesUsuario;
      });
  }

  findById(id_institucion_usuario: number): Promise<InstitucionUsuario> {
    return this.informacionInstitucionUsuarioView
      .findOne({ where: { id_institucion_usuario } })
      .then((infoInstitucionUsuario) => {
        if (!infoInstitucionUsuario)
          throw new ConflictException('No existe este id institución usuario.');
        return this.viewToInstitucionUsuario(infoInstitucionUsuario);
      });
  }

  findByIdUsuarioIdInstitucion(
    usuario: Usuario,
    institucion: Institucion,
  ): Promise<InstitucionUsuario> {
    return this.informacionInstitucionUsuarioView
      .findOne({
        where: {
          id_institucion: institucion.id_institucion,
          id_usuario: usuario.id_usuario,
        },
      })
      .then((infoInstitucionUsuario) =>
        this.viewToInstitucionUsuario(infoInstitucionUsuario),
      );
  }

  findByIdUsuarioIdInstitucionCarrera(
    usuario: Usuario,
    institucionCarrera: InstitucionCarrera,
  ): Promise<InstitucionUsuario> {
    return this.informacionInstitucionUsuarioView
      .findOne({
        where: {
          id_institucion_carrera: institucionCarrera.id_institucion_carrera,
          id_usuario: usuario.id_usuario,
        },
      })
      .then((infoInstitucionUsuario) =>
        this.viewToInstitucionUsuario(infoInstitucionUsuario),
      );
  }

  findFullInfoAllByIdUsuario(
    usuario: Usuario,
    activosSinMulta = true,
  ): Promise<InstitucionUsuario[]> {
    const busqueda: FindOptionsWhere<FullInformacionInstitucionUsuarioView> = {
      id_usuario: usuario.id_usuario,
    };

    if (activosSinMulta) {
      busqueda.activo = 1;
      busqueda.multa = 0;
    }
    return this.fullInformacionInstitucionUsuarioView
      .find({ where: busqueda })
      .then((infoInstitucionesUsuario) => {
        const institucionesUsuario: InstitucionUsuario[] = [];

        for (let i = 0; i < infoInstitucionesUsuario.length; i++)
          institucionesUsuario.push(
            this.fullViewToInstitucionUsuario(infoInstitucionesUsuario[i]),
          );
        return institucionesUsuario;
      });
  }

  findFullInfoAllByIdUsuarioIdInstitucion(
    usuario: Usuario,
    institucion: Institucion,
  ): Promise<InstitucionUsuario[]> {
    return this.fullInformacionInstitucionUsuarioView
      .find({
        where: {
          id_institucion: institucion.id_institucion,
          id_usuario: usuario.id_usuario,
        },
      })
      .then((infoInstitucionesUsuario) => {
        const institucionesUsuario: InstitucionUsuario[] = [];

        // Si el array esta vacio es que no pertenece a la institución
        if (infoInstitucionesUsuario.length === 0)
          throw new ConflictException(
            'Este usuario no pertenece a tu institución.',
          );
        for (let i = 0; i < infoInstitucionesUsuario.length; i++)
          institucionesUsuario.push(
            this.fullViewToInstitucionUsuario(infoInstitucionesUsuario[i]),
          );
        return institucionesUsuario;
      });
  }

  findFullInfoByIdUsuarioIdInstitucionCarrera(
    usuario: Usuario,
    institucionCarrera: InstitucionCarrera,
  ): Promise<InstitucionUsuario> {
    return this.fullInformacionInstitucionUsuarioView
      .findOne({
        where: {
          id_institucion_carrera: institucionCarrera.id_institucion_carrera,
          id_usuario: usuario.id_usuario,
        },
      })
      .then((infoInstitucionUsuario) => {
        if (!infoInstitucionUsuario || infoInstitucionUsuario.activo === 0)
          throw new ConflictException(
            'Verificar sus datos, cualquier duda asistir al módulo PC PUMA.',
          );

        const institucionUsuario = this.fullViewToInstitucionUsuario(
          infoInstitucionUsuario,
        );

        institucionUsuario.usuario = usuario;
        return institucionUsuario;
      });
  }

  fullViewToInstitucionUsuario(
    infoInstitucionUsuario: FullInformacionInstitucionUsuarioView,
  ): InstitucionUsuario {
    return this.repository.create({
      id_institucion_usuario: infoInstitucionUsuario.id_institucion_usuario,
      activo: infoInstitucionUsuario.activo === 1,
      multa: infoInstitucionUsuario.multa === 1,
      institucionCarrera: {
        id_institucion_carrera: infoInstitucionUsuario.id_institucion_carrera,
        carrera: {
          id_carrera: infoInstitucionUsuario.id_carrera,
          carrera: infoInstitucionUsuario.carrera,
        },
        institucion: {
          id_institucion: infoInstitucionUsuario.id_institucion,
          institucion: infoInstitucionUsuario.institucion,
        },
      },
    });
  }

  save(institucionUsuario: InstitucionUsuario): Promise<InstitucionUsuario> {
    return this.repository.save(institucionUsuario);
  }

  update(attrs: Partial<InstitucionUsuario>): Promise<{ message: string }> {
    return this.findById(attrs.id_institucion_usuario)
      .then((institucionUsuario) => {
        // Asignamos valores enviados al objeto
        Object.assign(institucionUsuario, attrs);
        // Guardamos cambios
        return this.repository.save(institucionUsuario);
      })
      .then((_) => ({ message: 'Se guardaron los cambios correctamente.' }));
  }

  updateMulta(
    institucion: Institucion,
    usuario: Usuario,
    status: boolean,
  ): Promise<{ message: string }> {
    return this.findAllByIdUsuarioIdInstitucion(usuario, institucion).then(
      async (institucionesUsuario) => {
        // Recorremos el array de instituciones usuario
        for (let i = 0; i < institucionesUsuario.length; i++) {
          // Activamos o deactivamos dependiendo del caso
          institucionesUsuario[i].multa = status;
          // Guardamos cambios
          await this.repository.save(institucionesUsuario[i]);
        }
        return { message: 'Se multó correctamente a este usuario.' };
      },
    );
  }

  viewToInstitucionUsuario(
    infoInstitucionUsuario: InformacionInstitucionUsuarioView,
  ): InstitucionUsuario {
    if (!infoInstitucionUsuario) return null;
    return this.repository.create({
      id_institucion_usuario: infoInstitucionUsuario.id_institucion_usuario,
      activo: infoInstitucionUsuario.activo === 1,
      multa: infoInstitucionUsuario.multa === 1,
      institucionCarrera: {
        id_institucion_carrera: infoInstitucionUsuario.id_institucion_carrera,
        carrera: { id_carrera: infoInstitucionUsuario.id_carrera },
        institucion: { id_institucion: infoInstitucionUsuario.id_institucion },
      },
      usuario: { id_usuario: infoInstitucionUsuario.id_usuario },
    });
  }
}
